@@ -3,13 +3,12 @@
 const {
   ArrayBufferPrototypeGetByteLength,
   ArrayFrom,
-  ArrayIsArray,
   ArrayPrototypePush,
   ArrayPrototypeSlice,
   ArrayPrototypeSort,
   Error,
+  ErrorCaptureStackTrace,
   FunctionPrototypeCall,
-  ObjectCreate,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectGetOwnPropertyDescriptor,
@@ -34,6 +33,7 @@ const {
   SafeMap,
   SafeSet,
   SafeWeakMap,
+  SafeWeakRef,
   StringPrototypeReplace,
   StringPrototypeToLowerCase,
   StringPrototypeToUpperCase,
@@ -44,26 +44,27 @@ const {
 } = primordials;
 
 const {
-  hideStackFrames,
   codes: {
     ERR_NO_CRYPTO,
     ERR_UNKNOWN_SIGNAL,
   },
+  isErrorStackTraceLimitWritable,
   uvErrmapGet,
   overrideStackTrace,
 } = require('internal/errors');
 const { signals } = internalBinding('constants').os;
 const {
   isArrayBufferDetached: _isArrayBufferDetached,
+  guessHandleType: _guessHandleType,
   privateSymbols: {
     arrow_message_private_symbol,
     decorated_private_symbol,
   },
   sleep: _sleep,
-  toUSVString: _toUSVString,
 } = internalBinding('util');
 const { isNativeError } = internalBinding('types');
 const { getOptionValue } = require('internal/options');
+const { encodings } = internalBinding('string_decoder');
 
 const noCrypto = !process.versions.openssl;
 
@@ -71,18 +72,6 @@ const experimentalWarnings = new SafeSet();
 
 const colorRegExp = /\u001b\[\d\d?m/g; // eslint-disable-line no-control-regex
 
-const unpairedSurrogateRe =
-  /(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])/;
-function toUSVString(val) {
-  const str = `${val}`;
-  // As of V8 5.5, `str.search()` (and `unpairedSurrogateRe[@@search]()`) are
-  // slower than `unpairedSurrogateRe.exec()`.
-  const match = RegExpPrototypeExec(unpairedSurrogateRe, str);
-  if (!match)
-    return str;
-  return _toUSVString(str, match.index);
-}
-
 let uvBinding;
 
 function lazyUv() {
@@ -153,10 +142,6 @@ function pendingDeprecate(fn, msg, code) {
 // Returns a modified function which warns once by default.
 // If --no-deprecation is set, then it is a no-op.
 function deprecate(fn, msg, code, useEmitSync) {
-  if (process.noDeprecation === true) {
-    return fn;
-  }
-
   // Lazy-load to avoid a circular dependency.
   if (validateString === undefined)
     ({ validateString } = require('internal/validators'));
@@ -169,7 +154,10 @@ function deprecate(fn, msg, code, useEmitSync) {
   );
 
   function deprecated(...args) {
-    emitDeprecationWarning();
+    // TODO(joyeecheung): use getOptionValue('--no-deprecation') instead.
+    if (!process.noDeprecation) {
+      emitDeprecationWarning();
+    }
     if (new.target) {
       return ReflectConstruct(fn, args, new.target);
     }
@@ -218,13 +206,13 @@ function slowCases(enc) {
     case 4:
       if (enc === 'UTF8') return 'utf8';
       if (enc === 'ucs2' || enc === 'UCS2') return 'utf16le';
-      enc = `${enc}`.toLowerCase();
+      enc = StringPrototypeToLowerCase(enc);
       if (enc === 'utf8') return 'utf8';
       if (enc === 'ucs2') return 'utf16le';
       break;
     case 3:
       if (enc === 'hex' || enc === 'HEX' ||
-          `${enc}`.toLowerCase() === 'hex')
+      StringPrototypeToLowerCase(enc) === 'hex')
         return 'hex';
       break;
     case 5:
@@ -233,7 +221,7 @@ function slowCases(enc) {
       if (enc === 'UTF-8') return 'utf8';
       if (enc === 'ASCII') return 'ascii';
       if (enc === 'UCS-2') return 'utf16le';
-      enc = `${enc}`.toLowerCase();
+      enc = StringPrototypeToLowerCase(enc);
       if (enc === 'utf-8') return 'utf8';
       if (enc === 'ascii') return 'ascii';
       if (enc === 'ucs-2') return 'utf16le';
@@ -243,23 +231,23 @@ function slowCases(enc) {
       if (enc === 'latin1' || enc === 'binary') return 'latin1';
       if (enc === 'BASE64') return 'base64';
       if (enc === 'LATIN1' || enc === 'BINARY') return 'latin1';
-      enc = `${enc}`.toLowerCase();
+      enc = StringPrototypeToLowerCase(enc);
       if (enc === 'base64') return 'base64';
       if (enc === 'latin1' || enc === 'binary') return 'latin1';
       break;
     case 7:
       if (enc === 'utf16le' || enc === 'UTF16LE' ||
-          `${enc}`.toLowerCase() === 'utf16le')
+      StringPrototypeToLowerCase(enc) === 'utf16le')
         return 'utf16le';
       break;
     case 8:
       if (enc === 'utf-16le' || enc === 'UTF-16LE' ||
-        `${enc}`.toLowerCase() === 'utf-16le')
+      StringPrototypeToLowerCase(enc) === 'utf-16le')
         return 'utf16le';
       break;
     case 9:
       if (enc === 'base64url' || enc === 'BASE64URL' ||
-          `${enc}`.toLowerCase() === 'base64url')
+      StringPrototypeToLowerCase(enc) === 'base64url')
         return 'base64url';
       break;
     default:
@@ -267,11 +255,20 @@ function slowCases(enc) {
   }
 }
 
-function emitExperimentalWarning(feature) {
+/**
+ * @param {string} feature Feature name used in the warning message
+ * @param {string} messagePrefix Prefix of the warning message
+ * @param {string} code See documentation of process.emitWarning
+ * @param {string} ctor See documentation of process.emitWarning
+ */
+function emitExperimentalWarning(feature, messagePrefix, code, ctor) {
   if (experimentalWarnings.has(feature)) return;
-  const msg = `${feature} is an experimental feature and might change at any time`;
   experimentalWarnings.add(feature);
-  process.emitWarning(msg, 'ExperimentalWarning');
+  let msg = `${feature} is an experimental feature and might change at any time`;
+  if (messagePrefix) {
+    msg = messagePrefix + msg;
+  }
+  process.emitWarning(msg, 'ExperimentalWarning', code, ctor);
 }
 
 function filterDuplicateStrings(items, low) {
@@ -322,7 +319,7 @@ function getSignalsToNamesMapping() {
   if (signalsToNamesMapping !== undefined)
     return signalsToNamesMapping;
 
-  signalsToNamesMapping = ObjectCreate(null);
+  signalsToNamesMapping = { __proto__: null };
   for (const key in signals) {
     signalsToNamesMapping[signals[key]] = key;
   }
@@ -484,18 +481,22 @@ function spliceOne(list, index) {
   list.pop();
 }
 
-const kNodeModulesRE = /^(.*)[\\/]node_modules[\\/]/;
+const kNodeModulesRE = /^(?:.*)[\\/]node_modules[\\/]/;
 
-let getStructuredStack;
+function isUnderNodeModules(filename) {
+  return filename && (RegExpPrototypeExec(kNodeModulesRE, filename) !== null);
+}
 
-function isInsideNodeModules() {
-  if (getStructuredStack === undefined) {
+let getStructuredStackImpl;
+
+function lazyGetStructuredStack() {
+  if (getStructuredStackImpl === undefined) {
     // Lazy-load to avoid a circular dependency.
     const { runInNewContext } = require('vm');
     // Use `runInNewContext()` to get something tamper-proof and
-    // side-effect-free. Since this is currently only used for a deprecated API,
-    // the perf implications should be okay.
-    getStructuredStack = runInNewContext(`(function() {
+    // side-effect-free. Since this is currently only used for a deprecated API
+    // and module mocking, the perf implications should be okay.
+    getStructuredStackImpl = runInNewContext(`(function() {
       try { Error.stackTraceLimit = Infinity; } catch {}
       return function structuredStack() {
         const e = new Error();
@@ -505,29 +506,24 @@ function isInsideNodeModules() {
     })()`, { overrideStackTrace }, { filename: 'structured-stack' });
   }
 
-  const stack = getStructuredStack();
-
-  // Iterate over all stack frames and look for the first one not coming
-  // from inside Node.js itself:
-  if (ArrayIsArray(stack)) {
-    for (const frame of stack) {
-      const filename = frame.getFileName();
-      // If a filename does not start with / or contain \,
-      // it's likely from Node.js core.
-      if (RegExpPrototypeExec(/^\/|\\/, filename) === null)
-        continue;
-      return RegExpPrototypeExec(kNodeModulesRE, filename) !== null;
-    }
-  }
-  return false;
+  return getStructuredStackImpl;
 }
 
-function once(callback) {
+function getStructuredStack() {
+  const getStructuredStackImpl = lazyGetStructuredStack();
+
+  return getStructuredStackImpl();
+}
+
+function once(callback, { preserveReturnValue = false } = kEmptyObject) {
   let called = false;
+  let returnValue;
   return function(...args) {
-    if (called) return;
+    if (called) return returnValue;
     called = true;
-    return ReflectApply(callback, this, args);
+    const result = ReflectApply(callback, this, args);
+    returnValue = preserveReturnValue ? result : undefined;
+    return result;
   };
 }
 
@@ -575,6 +571,27 @@ function exposeInterface(target, name, interfaceObject) {
   });
 }
 
+// https://heycam.github.io/webidl/#es-namespaces
+function exposeNamespace(target, name, namespaceObject) {
+  ObjectDefineProperty(target, name, {
+    __proto__: null,
+    writable: true,
+    enumerable: false,
+    configurable: true,
+    value: namespaceObject,
+  });
+}
+
+function exposeGetterAndSetter(target, name, getter, setter = undefined) {
+  ObjectDefineProperty(target, name, {
+    __proto__: null,
+    enumerable: false,
+    configurable: true,
+    get: getter,
+    set: setter,
+  });
+}
+
 function defineLazyProperties(target, id, keys, enumerable = true) {
   const descriptors = { __proto__: null };
   let mod;
@@ -615,7 +632,7 @@ function defineLazyProperties(target, id, keys, enumerable = true) {
   ObjectDefineProperties(target, descriptors);
 }
 
-function defineReplaceableLazyAttribute(target, id, keys, writable = true) {
+function defineReplaceableLazyAttribute(target, id, keys, writable = true, check) {
   let mod;
   for (let i = 0; i < keys.length; i++) {
     const key = keys[i];
@@ -623,6 +640,9 @@ function defineReplaceableLazyAttribute(target, id, keys, writable = true) {
     let setterCalled = false;
 
     function get() {
+      if (check !== undefined) {
+        FunctionPrototypeCall(check, this);
+      }
       if (setterCalled) {
         return value;
       }
@@ -665,19 +685,28 @@ const lazyDOMExceptionClass = () => {
   return _DOMException;
 };
 
-const lazyDOMException = hideStackFrames((message, name) => {
+const lazyDOMException = (message, name) => {
   _DOMException ??= internalBinding('messaging').DOMException;
+  if (isErrorStackTraceLimitWritable()) {
+    const limit = Error.stackTraceLimit;
+    Error.stackTraceLimit = 0;
+    const ex = new _DOMException(message, name);
+    Error.stackTraceLimit = limit;
+    ErrorCaptureStackTrace(ex, lazyDOMException);
+    return ex;
+  }
   return new _DOMException(message, name);
-});
 
-const kEnumerableProperty = ObjectCreate(null);
+};
+
+const kEnumerableProperty = { __proto__: null };
 kEnumerableProperty.enumerable = true;
 ObjectFreeze(kEnumerableProperty);
 
-const kEmptyObject = ObjectFreeze(ObjectCreate(null));
+const kEmptyObject = ObjectFreeze({ __proto__: null });
 
 function filterOwnProperties(source, keys) {
-  const filtered = ObjectCreate(null);
+  const filtered = { __proto__: null };
   for (let i = 0; i < keys.length; i++) {
     const key = keys[i];
     if (ObjectPrototypeHasOwnProperty(source, key)) {
@@ -796,6 +825,49 @@ function setupCoverageHooks(dir) {
   return coverageDirectory;
 }
 
+
+const handleTypes = ['TCP', 'TTY', 'UDP', 'FILE', 'PIPE', 'UNKNOWN'];
+function guessHandleType(fd) {
+  const type = _guessHandleType(fd);
+  return handleTypes[type];
+}
+
+class WeakReference {
+  #weak = null;
+  #strong = null;
+  #refCount = 0;
+  constructor(object) {
+    this.#weak = new SafeWeakRef(object);
+  }
+
+  incRef() {
+    this.#refCount++;
+    if (this.#refCount === 1) {
+      const derefed = this.#weak.deref();
+      if (derefed !== undefined) {
+        this.#strong = derefed;
+      }
+    }
+    return this.#refCount;
+  }
+
+  decRef() {
+    this.#refCount--;
+    if (this.#refCount === 0) {
+      this.#strong = null;
+    }
+    return this.#refCount;
+  }
+
+  get() {
+    return this.#weak.deref();
+  }
+}
+
+const encodingsMap = { __proto__: null };
+for (let i = 0; i < encodings.length; ++i)
+  encodingsMap[encodings[i]] = i;
+
 module.exports = {
   getLazy,
   assertCrypto,
@@ -809,18 +881,23 @@ module.exports = {
   defineReplaceableLazyAttribute,
   deprecate,
   emitExperimentalWarning,
+  encodingsMap,
   exposeInterface,
   exposeLazyInterfaces,
+  exposeNamespace,
+  exposeGetterAndSetter,
   filterDuplicateStrings,
   filterOwnProperties,
   getConstructorOf,
   getCWDURL,
   getInternalGlobal,
+  getStructuredStack,
   getSystemErrorMap,
   getSystemErrorName,
+  guessHandleType,
   isArrayBufferDetached,
   isError,
-  isInsideNodeModules,
+  isUnderNodeModules,
   join,
   lazyDOMException,
   lazyDOMExceptionClass,
@@ -833,9 +910,16 @@ module.exports = {
   sleep,
   spliceOne,
   setupCoverageHooks,
-  toUSVString,
   removeColors,
 
+  // Define Symbol.dispose and Symbol.asyncDispose
+  // Until these are defined by the environment.
+  // TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in primordials.
+  // eslint-disable-next-line node-core/prefer-primordials
+  SymbolDispose: Symbol.dispose || SymbolFor('nodejs.dispose'),
+  // eslint-disable-next-line node-core/prefer-primordials
+  SymbolAsyncDispose: Symbol.asyncDispose || SymbolFor('nodejs.asyncDispose'),
+
   // Symbol used to customize promisify conversion
   customPromisifyArgs: kCustomPromisifyArgsSymbol,
 
@@ -852,4 +936,5 @@ module.exports = {
   kEnumerableProperty,
   setOwnProperty,
   pendingDeprecate,
+  WeakReference,
 };
